@@ -10,6 +10,7 @@
  **********************************************************************/
 #include "stdafx.h"
 #include "GeographicLib/GeodesicExact.hpp"
+#include "GeographicLib/GeodesicLineExact.hpp"
 #include "GeodesicExact.h"
 #include "GeodesicLineExact.h"
 
@@ -476,6 +477,38 @@ GeodesicLineExact^ GeodesicExact::Line(double lat1, double lon1, double azi1,
     return gcnew GeodesicLineExact( this, lat1, lon1, azi1, caps );
 }
 
+//*****************************************************************************
+GeodesicLineExact^ GeodesicExact::InverseLine(double lat1, double lon1,
+    double lat2, double lon2, NETGeographicLib::Mask caps)
+{
+    return gcnew GeodesicLineExact(m_pGeodesicExact->InverseLine(
+        lat1, lon1, lat2, lon2, static_cast<unsigned>(caps)));
+}
+
+//*****************************************************************************
+GeodesicLineExact^ GeodesicExact::DirectLine(double lat1, double lon1,
+    double azi1, double s12, NETGeographicLib::Mask caps)
+{
+    return gcnew GeodesicLineExact(m_pGeodesicExact->DirectLine(
+        lat1, lon1, azi1, s12, static_cast<unsigned>(caps)));
+}
+
+//*****************************************************************************
+GeodesicLineExact^ GeodesicExact::ArcDirectLine(double lat1, double lon1,
+    double azi1, double a12, NETGeographicLib::Mask caps)
+{
+    return gcnew GeodesicLineExact(m_pGeodesicExact->ArcDirectLine(
+        lat1, lon1, azi1, a12, static_cast<unsigned>(caps)));
+}
+
+//*****************************************************************************
+GeodesicLineExact^ GeodesicExact::GenDirectLine(double lat1, double lon1,
+    double azi1, bool arcmode, double s12_a12, NETGeographicLib::Mask caps)
+{
+    return gcnew GeodesicLineExact(m_pGeodesicExact->GenDirectLine(
+        lat1, lon1, azi1, arcmode, s12_a12, static_cast<unsigned>(caps)));
+}
+
 //*****************************************************************************
 double GeodesicExact::MajorRadius::get()
 { return m_pGeodesicExact->MajorRadius(); }
