@@ -45,6 +45,19 @@ GeodesicLine::GeodesicLine( Geodesic^ g, double lat1, double lon1, double azi1,
     }
 }
 
+//*****************************************************************************
+GeodesicLine::GeodesicLine(const GeographicLib::GeodesicLine& gl)
+{
+    try
+    {
+        m_pGeodesicLine = new GeographicLib::GeodesicLine(gl);
+    }
+    catch (std::bad_alloc)
+    {
+        throw gcnew GeographicErr(BADALLOC);
+    }
+}
+
 //*****************************************************************************
 GeodesicLine::GeodesicLine(double lat1, double lon1, double azi1,
                            NETGeographicLib::Mask caps)
@@ -341,6 +354,14 @@ double GeodesicLine::MajorRadius::get()
 double GeodesicLine::Flattening::get()
 { return m_pGeodesicLine->Flattening(); }
 
+//*****************************************************************************
+double GeodesicLine::Distance::get()
+{ return m_pGeodesicLine->Distance(); }
+
+//*****************************************************************************
+double GeodesicLine::Arc::get()
+{ return m_pGeodesicLine->Arc(); }
+
 //*****************************************************************************
 NETGeographicLib::Mask GeodesicLine::Capabilities()
 { return static_cast<NETGeographicLib::Mask>(m_pGeodesicLine->Capabilities()); }
@@ -348,3 +369,37 @@ NETGeographicLib::Mask GeodesicLine::Capabilities()
 //*****************************************************************************
 bool GeodesicLine::Capabilities(GeodesicLine::mask testcaps)
 { return m_pGeodesicLine->Capabilities( static_cast<unsigned>(testcaps) ); }
+
+//*****************************************************************************
+void GeodesicLine::SetDistance(double s13)
+{ m_pGeodesicLine->SetDistance(s13); }
+
+//*****************************************************************************
+void GeodesicLine::SetArc(double a13)
+{ m_pGeodesicLine->SetArc(a13); }
+
+//*****************************************************************************
+void GeodesicLine::GenSetDistance(bool arcmode, double s13_a13)
+{ m_pGeodesicLine->GenSetDistance(arcmode, s13_a13); }
+
+//*****************************************************************************
+void GeodesicLine::AzimuthSinCos(double% sazi1, double% cazi1)
+{
+    double x1, x2;
+    m_pGeodesicLine->Azimuth(x1, x2);
+    sazi1 = x1;
+    cazi1 = x2;
+}
+
+//*****************************************************************************
+void GeodesicLine::EquatorialAzimuthSinCos(double% sazi0, double% cazi0)
+{
+    double x1, x2;
+    m_pGeodesicLine->EquatorialAzimuth(x1, x2);
+    sazi0 = x1;
+    cazi0 = x2;
+}
+
+//*****************************************************************************
+double GeodesicLine::GenDistance(bool arcmode)
+{ return m_pGeodesicLine->GenDistance(arcmode); }
