@@ -44,7 +44,7 @@ namespace NETGeographicLib
    *   J. Geodesy <b>87</b>, 43--55 (2013);
    *   DOI: <a href="https://dx.doi.org/10.1007/s00190-012-0578-z">
    *   10.1007/s00190-012-0578-z</a>;
-   *   addenda: <a href="http://geographiclib.sf.net/geod-addenda.html">
+   *   addenda: <a href="http://geographiclib.sourceforge.net/geod-addenda.html">
    *   geod-addenda.html</a>.
    * .
    * For more information on geodesics see \ref geodesic.
@@ -61,18 +61,22 @@ namespace NETGeographicLib
    *
    * The following functions are implemented as properties:
    * Latitude, Longitude, Azimuth, EquatorialAzimuth, EquatorialArc,
-   * MajorRadius, and Flattening.
+   * MajorRadius, Distance, Arc, and Flattening.
    *
    * The constructors, Capabilities, and GenPosition functions accept the
    * "capabilities mask" as a NETGeographicLib::Mask rather than an
    * unsigned.  The Capabilities function returns a  NETGeographicLib::Mask
    * rather than an unsigned.
+   *
+   * The overloaded Azimuth and EquatorialAzimuth functions that return
+   * the sin and cosine terms have been renamed AzimuthSinCos and
+   * EquatorialAzimuthSinCos, repectively.
    **********************************************************************/
     public ref class GeodesicLine
     {
         private:
         // pointer to the unmanaged GeographicLib::GeodesicLine.
-        const GeographicLib::GeodesicLine* m_pGeodesicLine;
+        GeographicLib::GeodesicLine* m_pGeodesicLine;
 
         // The finalizer frees the unmanaged memory when this object is destroyed.
         !GeodesicLine(void);
@@ -135,12 +139,10 @@ namespace NETGeographicLib
           **********************************************************************/
          AREA          = 1U<<14 | unsigned(captype::CAP_C4),
          /**
-          * Unroll \e lon2 in the direct calculation.  (This flag used to be
-          * called LONG_NOWRAP.)
+          * Unroll \e lon2 in the direct calculation.
           * @hideinitializer
           **********************************************************************/
          LONG_UNROLL   = 1U<<15,
-         LONG_NOWRAP   = LONG_UNROLL,
          /**
           * All capabilities, calculate everything.  (LONG_UNROLL is not
           * included in this mask.)
@@ -198,6 +200,12 @@ namespace NETGeographicLib
          **********************************************************************/
         GeodesicLine(double lat1, double lon1, double azi1,
                      NETGeographicLib::Mask caps);
+
+        /**
+        * A constructoe that accepts a reference to an unmanages GeodesicLin.
+        * FOR INTERNAL USE ONLY
+        **********************************************************************/
+        GeodesicLine(const GeographicLib::GeodesicLine& gl);
         ///@}
 
         /**
@@ -488,6 +496,77 @@ namespace NETGeographicLib
 
         ///@}
 
+        /** \name Setting point 3
+        **********************************************************************/
+        ///@{
+
+        /**
+        * Specify position of point 3 in terms of distance.
+        *
+        * @param[in] s13 the distance from point 1 to point 3 (meters); it
+        *   can be negative.
+        *
+        * This is only useful if the GeodesicLine object has been constructed
+        * with \e caps |= GeodesicLine::DISTANCE_IN.
+        **********************************************************************/
+        void SetDistance(double s13);
+
+        /**
+        * Specify position of point 3 in terms of arc length.
+        *
+        * @param[in] a13 the arc length from point 1 to point 3 (degrees); it
+        *   can be negative.
+        *
+        * The distance \e s13 is only set if the GeodesicLine object has been
+        * constructed with \e caps |= GeodesicLine::DISTANCE.
+        **********************************************************************/
+        void SetArc(double a13);
+
+        /**
+        * Specify position of point 3 in terms of either distance or arc length.
+        *
+        * @param[in] arcmode boolean flag determining the meaning of the second
+        *   parameter; if \e arcmode is false, then the GeodesicLine object must
+        *   have been constructed with \e caps |= GeodesicLine::DISTANCE_IN.
+        * @param[in] s13_a13 if \e arcmode is false, this is the distance from
+        *   point 1 to point 3 (meters); otherwise it is the arc length from
+        *   point 1 to point 3 (degrees); it can be negative.
+        **********************************************************************/
+        void GenSetDistance(bool arcmode, double s13_a13);
+        ///@}
+
+       /** \name Trigonometric accessor functions
+        **********************************************************************/
+        ///@{
+        /**
+        * The sine and cosine of \e azi1.
+        *
+        * @param[out] sazi1 the sine of \e azi1.
+        * @param[out] cazi1 the cosine of \e azi1.
+        **********************************************************************/
+        void AzimuthSinCos([System::Runtime::InteropServices::Out] double% sazi1,
+            [System::Runtime::InteropServices::Out] double% cazi1);
+
+        /**
+        * The sine and cosine of \e azi0.
+        *
+        * @param[out] sazi0 the sine of \e azi0.
+        * @param[out] cazi0 the cosine of \e azi0.
+        **********************************************************************/
+        void EquatorialAzimuthSinCos(
+            [System::Runtime::InteropServices::Out] double% sazi0,
+            [System::Runtime::InteropServices::Out] double% cazi0);
+
+        /**
+        * The distance or arc length to point 3.
+        *
+        * @param[in] arcmode boolean flag determining the meaning of returned
+        *   value.
+        * @return \e s13 if \e arcmode is false; \e a13 if \e arcmode is true.
+        **********************************************************************/
+        double GenDistance(bool arcmode);
+        ///@}
+
         /** \name Inspector functions
          **********************************************************************/
         ///@{
@@ -531,6 +610,16 @@ namespace NETGeographicLib
          **********************************************************************/
         property double Flattening { double get(); }
 
+        /**
+        * @return \e s13, the distance to point 3 (meters).
+        **********************************************************************/
+        property double Distance { double get(); }
+
+        /**
+        * @return \e a13, the arc length to point 3 (degrees).
+        **********************************************************************/
+        property double Arc { double get(); }
+
         /**
          * @return \e caps the computational capabilities that this object was
          *   constructed with.  LATITUDE and AZIMUTH are always included.
