@@ -92,7 +92,7 @@ namespace NETGeographicLib
        * Do not wrap the \e lon2 in the direct calculation.
        * @hideinitializer
        **********************************************************************/
-      LONG_NOWRAP   = 1U<<15,
+      LONG_UNROLL   = 1U<<15,
       /**
        * All capabilities, calculate everything.
        * @hideinitializer
