@@ -2,7 +2,7 @@
  * \file GeodesicLine.hpp
  * \brief Header for GeographicLib::GeodesicLine class
  *
- * Copyright (c) Charles Karney (2009-2015) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2009-2016) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
@@ -20,10 +20,21 @@ namespace GeographicLib {
    *
    * GeodesicLine facilitates the determination of a series of points on a
    * single geodesic.  The starting point (\e lat1, \e lon1) and the azimuth \e
-   * azi1 are specified in the constructor.  GeodesicLine.Position returns the
-   * location of point 2 a distance \e s12 along the geodesic.  Alternatively
-   * GeodesicLine.ArcPosition gives the position of point 2 an arc length \e
-   * a12 along the geodesic.
+   * azi1 are specified in the constructor; alternatively, the Geodesic::Line
+   * method can be used to create a GeodesicLine.  GeodesicLine.Position
+   * returns the location of point 2 a distance \e s12 along the geodesic.  In
+   * addition, GeodesicLine.ArcPosition gives the position of point 2 an arc
+   * length \e a12 along the geodesic.
+   *
+   * You can register the position of a reference point 3 a distance (arc
+   * length), \e s13 (\e a13) along the geodesic with the
+   * GeodesicLine.SetDistance (GeodesicLine.SetArc) functions.  Points a
+   * fractional distance along the line can be found by providing, for example,
+   * 0.5 * Distance() as an argument to GeodesicLine.Position.  The
+   * Geodesic::InverseLine or Geodesic::DirectLine methods return GeodesicLine
+   * objects with point 3 set to the point 2 of the corresponding geodesic
+   * problem.  GeodesicLine objects created with the public constructor or with
+   * Geodesic::Line have \e s13 and \e a13 set to NaNs.
    *
    * The default copy constructor and assignment operators work with this
    * class.  Similarly, a vector can be used to hold GeodesicLine objects.
@@ -44,7 +55,7 @@ namespace GeographicLib {
    *   J. Geodesy <b>87</b>, 43--55 (2013);
    *   DOI: <a href="https://dx.doi.org/10.1007/s00190-012-0578-z">
    *   10.1007/s00190-012-0578-z</a>;
-   *   addenda: <a href="http://geographiclib.sf.net/geod-addenda.html">
+   *   addenda: <a href="http://geographiclib.sourceforge.net/geod-addenda.html">
    *   geod-addenda.html</a>.
    * .
    * For more information on geodesics see \ref geodesic.
@@ -71,11 +82,21 @@ namespace GeographicLib {
     real _a, _f, _b, _c2, _f1, _salp0, _calp0, _k2,
       _salp1, _calp1, _ssig1, _csig1, _dn1, _stau1, _ctau1, _somg1, _comg1,
       _A1m1, _A2m1, _A3c, _B11, _B21, _B31, _A4, _B41;
+    real _a13, _s13;
     // index zero elements of _C1a, _C1pa, _C2a, _C3a are unused
     real _C1a[nC1_ + 1], _C1pa[nC1p_ + 1], _C2a[nC2_ + 1], _C3a[nC3_],
       _C4a[nC4_];    // all the elements of _C4a are used
     unsigned _caps;
 
+    void LineInit(const Geodesic& g,
+                  real lat1, real lon1,
+                  real azi1, real salp1, real calp1,
+                  unsigned caps);
+    GeodesicLine(const Geodesic& g,
+                 real lat1, real lon1,
+                 real azi1, real salp1, real calp1,
+                 unsigned caps, bool arcmode, real s13_a13);
+
     enum captype {
       CAP_NONE = Geodesic::CAP_NONE,
       CAP_C1   = Geodesic::CAP_C1,
@@ -147,14 +168,10 @@ namespace GeographicLib {
        **********************************************************************/
       AREA          = Geodesic::AREA,
       /**
-       * Unroll \e lon2 in the direct calculation.  (This flag used to be
-       * called LONG_NOWRAP.)
+       * Unroll \e lon2 in the direct calculation.
        * @hideinitializer
        **********************************************************************/
       LONG_UNROLL   = Geodesic::LONG_UNROLL,
-      /// \cond SKIP
-      LONG_NOWRAP   = LONG_UNROLL,
-      /// \endcond
       /**
        * All capabilities, calculate everything.  (LONG_UNROLL is not
        * included in this mask.)
@@ -225,7 +242,7 @@ namespace GeographicLib {
      * Compute the position of point 2 which is a distance \e s12 (meters) from
      * point 1.
      *
-     * @param[in] s12 distance between point 1 and point 2 (meters); it can be
+     * @param[in] s12 distance from point 1 to point 2 (meters); it can be
      *   negative.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
@@ -244,7 +261,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The values of \e lon2 and \e azi2 returned are in the range
      * [&minus;180&deg;, 180&deg;).
@@ -330,7 +347,6 @@ namespace GeographicLib {
                          REDUCEDLENGTH | GEODESICSCALE,
                          lat2, lon2, azi2, t, m12, M12, M21, t);
     }
-
     ///@}
 
     /** \name Position in terms of arc length
@@ -341,14 +357,14 @@ namespace GeographicLib {
      * Compute the position of point 2 which is an arc length \e a12 (degrees)
      * from point 1.
      *
-     * @param[in] a12 arc length between point 1 and point 2 (degrees); it can
+     * @param[in] a12 arc length from point 1 to point 2 (degrees); it can
      *   be negative.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
      *   GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -465,8 +481,8 @@ namespace GeographicLib {
      * GeodesicLine::ArcPosition are defined in terms of this function.
      *
      * @param[in] arcmode boolean flag determining the meaning of the second
-     *   parameter; if arcmode is false, then the GeodesicLine object must have
-     *   been constructed with \e caps |= GeodesicLine::DISTANCE_IN.
+     *   parameter; if \e arcmode is false, then the GeodesicLine object must
+     *   have been constructed with \e caps |= GeodesicLine::DISTANCE_IN.
      * @param[in] s12_a12 if \e arcmode is false, this is the distance between
      *   point 1 and point 2 (meters); otherwise it is the arc length between
      *   point 1 and point 2 (degrees); it can be negative.
@@ -477,7 +493,7 @@ namespace GeographicLib {
      *   GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -492,7 +508,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The GeodesicLine::mask values possible for \e outmask are
      * - \e outmask |= GeodesicLine::LATITUDE for the latitude \e lat2;
@@ -521,7 +537,45 @@ namespace GeographicLib {
                            real& lat2, real& lon2, real& azi2,
                            real& s12, real& m12, real& M12, real& M21,
                            real& S12) const;
+    ///@}
 
+    /** \name Setting point 3
+     **********************************************************************/
+    ///@{
+
+    /**
+     * Specify position of point 3 in terms of distance.
+     *
+     * @param[in] s13 the distance from point 1 to point 3 (meters); it
+     *   can be negative.
+     *
+     * This is only useful if the GeodesicLine object has been constructed
+     * with \e caps |= GeodesicLine::DISTANCE_IN.
+     **********************************************************************/
+    void SetDistance(real s13);
+
+    /**
+     * Specify position of point 3 in terms of arc length.
+     *
+     * @param[in] a13 the arc length from point 1 to point 3 (degrees); it
+     *   can be negative.
+     *
+     * The distance \e s13 is only set if the GeodesicLine object has been
+     * constructed with \e caps |= GeodesicLine::DISTANCE.
+     **********************************************************************/
+    void SetArc(real a13);
+
+    /**
+     * Specify position of point 3 in terms of either distance or arc length.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of the second
+     *   parameter; if \e arcmode is false, then the GeodesicLine object must
+     *   have been constructed with \e caps |= GeodesicLine::DISTANCE_IN.
+     * @param[in] s13_a13 if \e arcmode is false, this is the distance from
+     *   point 1 to point 3 (meters); otherwise it is the arc length from
+     *   point 1 to point 3 (degrees); it can be negative.
+     **********************************************************************/
+    void GenSetDistance(bool arcmode, real s13_a13);
     ///@}
 
     /** \name Inspector functions
@@ -551,15 +605,32 @@ namespace GeographicLib {
     Math::real Azimuth() const
     { return Init() ? _azi1 : Math::NaN(); }
 
+    /**
+     * The sine and cosine of \e azi1.
+     *
+     * @param[out] sazi1 the sine of \e azi1.
+     * @param[out] cazi1 the cosine of \e azi1.
+     **********************************************************************/
+    void Azimuth(real& sazi1, real& cazi1) const
+    { if (Init()) { sazi1 = _salp1; cazi1 = _calp1; } }
+
     /**
      * @return \e azi0 the azimuth (degrees) of the geodesic line as it crosses
      *   the equator in a northward direction.
      *
      * The result lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real EquatorialAzimuth() const {
-      return Init() ? Math::atan2d(_salp0, _calp0) : Math::NaN();
-    }
+    Math::real EquatorialAzimuth() const
+    { return Init() ? Math::atan2d(_salp0, _calp0) : Math::NaN(); }
+
+    /**
+     * The sine and cosine of \e azi0.
+     *
+     * @param[out] sazi0 the sine of \e azi0.
+     * @param[out] cazi0 the cosine of \e azi0.
+     **********************************************************************/
+    void EquatorialAzimuth(real& sazi0, real& cazi0) const
+    { if (Init()) { sazi0 = _salp0; cazi0 = _calp0; } }
 
     /**
      * @return \e a1 the arc length (degrees) between the northward equatorial
@@ -585,15 +656,6 @@ namespace GeographicLib {
     Math::real Flattening() const
     { return Init() ? _f : Math::NaN(); }
 
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the ellipsoid.
-     **********************************************************************/
-    Math::real InverseFlattening() const
-    { return Init() ? 1/_f : Math::NaN(); }
-    /// \endcond
-
     /**
      * @return \e caps the computational capabilities that this object was
      *   constructed with.  LATITUDE and AZIMUTH are always included.
@@ -601,6 +663,8 @@ namespace GeographicLib {
     unsigned Capabilities() const { return _caps; }
 
     /**
+     * Test what capabilities are available.
+     *
      * @param[in] testcaps a set of bitor'ed GeodesicLine::mask values.
      * @return true if the GeodesicLine object has all these capabilities.
      **********************************************************************/
@@ -608,6 +672,26 @@ namespace GeographicLib {
       testcaps &= OUT_ALL;
       return (_caps & testcaps) == testcaps;
     }
+
+    /**
+     * The distance or arc length to point 3.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of returned
+     *   value.
+     * @return \e s13 if \e arcmode is false; \e a13 if \e arcmode is true.
+     **********************************************************************/
+    Math::real GenDistance(bool arcmode) const
+    { return Init() ? (arcmode ? _a13 : _s13) : Math::NaN(); }
+
+    /**
+     * @return \e s13, the distance to point 3 (meters).
+     **********************************************************************/
+    Math::real Distance() const { return GenDistance(false); }
+
+    /**
+     * @return \e a13, the arc length to point 3 (degrees).
+     **********************************************************************/
+    Math::real Arc() const { return GenDistance(true); }
     ///@}
 
   };
