@@ -2,7 +2,7 @@
  * \file GeodesicLineExact.hpp
  * \brief Header for GeographicLib::GeodesicLineExact class
  *
- * Copyright (c) Charles Karney (2012-2015) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2012-2016) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
@@ -44,10 +44,20 @@ namespace GeographicLib {
       _salp1, _calp1, _ssig1, _csig1, _dn1, _stau1, _ctau1,
       _somg1, _comg1, _cchi1,
       _A4, _B41, _E0, _D0, _H0, _E1, _D1, _H1;
+    real _a13, _s13;
     real _C4a[nC4_];            // all the elements of _C4a are used
     EllipticFunction _E;
     unsigned _caps;
 
+    void LineInit(const GeodesicExact& g,
+                  real lat1, real lon1,
+                  real azi1, real salp1, real calp1,
+                  unsigned caps);
+    GeodesicLineExact(const GeodesicExact& g,
+                      real lat1, real lon1,
+                      real azi1, real salp1, real calp1,
+                      unsigned caps, bool arcmode, real s13_a13);
+
     enum captype {
       CAP_NONE = GeodesicExact::CAP_NONE,
       CAP_E    = GeodesicExact::CAP_E,
@@ -119,14 +129,10 @@ namespace GeographicLib {
        **********************************************************************/
       AREA          = GeodesicExact::AREA,
       /**
-       * Unroll \e lon2 in the direct calculation.  (This flag used to be
-       * called LONG_NOWRAP.)
+       * Unroll \e lon2 in the direct calculation.
        * @hideinitializer
        **********************************************************************/
       LONG_UNROLL = GeodesicExact::LONG_UNROLL,
-      /// \cond SKIP
-      LONG_NOWRAP   = LONG_UNROLL,
-      /// \endcond
       /**
        * All capabilities, calculate everything.  (LONG_UNROLL is not
        * included in this mask.)
@@ -199,7 +205,7 @@ namespace GeographicLib {
      * Compute the position of point 2 which is a distance \e s12 (meters)
      * from point 1.
      *
-     * @param[in] s12 distance between point 1 and point 2 (meters); it can be
+     * @param[in] s12 distance from point 1 to point 2 (meters); it can be
      *   signed.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
@@ -218,7 +224,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The values of \e lon2 and \e azi2 returned are in the range
      * [&minus;180&deg;, 180&deg;).
@@ -304,7 +310,6 @@ namespace GeographicLib {
                          REDUCEDLENGTH | GEODESICSCALE,
                          lat2, lon2, azi2, t, m12, M12, M21, t);
     }
-
     ///@}
 
     /** \name Position in terms of arc length
@@ -315,14 +320,14 @@ namespace GeographicLib {
      * Compute the position of point 2 which is an arc length \e a12 (degrees)
      * from point 1.
      *
-     * @param[in] a12 arc length between point 1 and point 2 (degrees); it can
+     * @param[in] a12 arc length from point 1 to point 2 (degrees); it can
      *   be signed.
      * @param[out] lat2 latitude of point 2 (degrees).
      * @param[out] lon2 longitude of point 2 (degrees); requires that the
      *   GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -439,8 +444,9 @@ namespace GeographicLib {
      * GeodesicLineExact::ArcPosition are defined in terms of this function.
      *
      * @param[in] arcmode boolean flag determining the meaning of the second
-     *   parameter; if arcmode is false, then the GeodesicLineExact object must
-     *   have been constructed with \e caps |= GeodesicLineExact::DISTANCE_IN.
+     *   parameter; if \e arcmode is false, then the GeodesicLineExact object
+     *   must have been constructed with \e caps |=
+     *   GeodesicLineExact::DISTANCE_IN.
      * @param[in] s12_a12 if \e arcmode is false, this is the distance between
      *   point 1 and point 2 (meters); otherwise it is the arc length between
      *   point 1 and point 2 (degrees); it can be signed.
@@ -451,7 +457,7 @@ namespace GeographicLib {
      *   GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::LONGITUDE.
      * @param[out] azi2 (forward) azimuth at point 2 (degrees).
-     * @param[out] s12 distance between point 1 and point 2 (meters); requires
+     * @param[out] s12 distance from point 1 to point 2 (meters); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::DISTANCE.
      * @param[out] m12 reduced length of geodesic (meters); requires that the
@@ -466,7 +472,7 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>); requires
      *   that the GeodesicLineExact object was constructed with \e caps |=
      *   GeodesicLineExact::AREA.
-     * @return \e a12 arc length of between point 1 and point 2 (degrees).
+     * @return \e a12 arc length from point 1 to point 2 (degrees).
      *
      * The GeodesicLineExact::mask values possible for \e outmask are
      * - \e outmask |= GeodesicLineExact::LATITUDE for the latitude \e lat2;
@@ -495,9 +501,47 @@ namespace GeographicLib {
                            real& lat2, real& lon2, real& azi2,
                            real& s12, real& m12, real& M12, real& M21,
                            real& S12) const;
-
     ///@}
 
+    /** \name Setting point 3
+     **********************************************************************/
+    ///@{
+
+    /**
+     * Specify position of point 3 in terms of distance.
+     *
+     * @param[in] s13 the distance from point 1 to point 3 (meters); it
+     *   can be negative.
+     *
+     * This is only useful if the GeodesicLineExact object has been constructed
+     * with \e caps |= GeodesicLineExact::DISTANCE_IN.
+     **********************************************************************/
+    void SetDistance(real s13);
+
+    /**
+     * Specify position of point 3 in terms of arc length.
+     *
+     * @param[in] a13 the arc length from point 1 to point 3 (degrees); it
+     *   can be negative.
+     *
+     * The distance \e s13 is only set if the GeodesicLineExact object has been
+     * constructed with \e caps |= GeodesicLineExact::DISTANCE.
+     **********************************************************************/
+    void SetArc(real a13);
+
+    /**
+     * Specify position of point 3 in terms of either distance or arc length.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of the second
+     *   parameter; if \e arcmode is false, then the GeodesicLineExact object
+     *   must have been constructed with \e caps |=
+     *   GeodesicLineExact::DISTANCE_IN.
+     * @param[in] s13_a13 if \e arcmode is false, this is the distance from
+     *   point 1 to point 3 (meters); otherwise it is the arc length from
+     *   point 1 to point 3 (degrees); it can be negative.
+     **********************************************************************/
+    void GenSetDistance(bool arcmode, real s13_a13);
+
     /** \name Inspector functions
      **********************************************************************/
     ///@{
@@ -525,16 +569,32 @@ namespace GeographicLib {
     Math::real Azimuth() const
     { return Init() ? _azi1 : Math::NaN(); }
 
+    /**
+     * The sine and cosine of \e azi1.
+     *
+     * @param[out] sazi1 the sine of \e azi1.
+     * @param[out] cazi1 the cosine of \e azi1.
+     **********************************************************************/
+    void Azimuth(real& sazi1, real& cazi1) const
+    { if (Init()) { sazi1 = _salp1; cazi1 = _calp1; } }
+
     /**
      * @return \e azi0 the azimuth (degrees) of the geodesic line as it crosses
      *   the equator in a northward direction.
      *
      * The result lies in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
-    Math::real EquatorialAzimuth() const {
-      using std::atan2;
-      return Init() ? Math::atan2d(_salp0, _calp0) : Math::NaN();
-    }
+    Math::real EquatorialAzimuth() const
+    { return Init() ? Math::atan2d(_salp0, _calp0) : Math::NaN(); }
+
+    /**
+     * The sine and cosine of \e azi0.
+     *
+     * @param[out] sazi0 the sine of \e azi0.
+     * @param[out] cazi0 the cosine of \e azi0.
+     **********************************************************************/
+    void EquatorialAzimuth(real& sazi0, real& cazi0) const
+    { if (Init()) { sazi0 = _salp0; cazi0 = _calp0; } }
 
     /**
      * @return \e a1 the arc length (degrees) between the northward equatorial
@@ -562,15 +622,6 @@ namespace GeographicLib {
     Math::real Flattening() const
     { return Init() ? _f : Math::NaN(); }
 
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the ellipsoid.
-     **********************************************************************/
-    Math::real InverseFlattening() const
-    { return Init() ? 1/_f : Math::NaN(); }
-    /// \endcond
-
     /**
      * @return \e caps the computational capabilities that this object was
      *   constructed with.  LATITUDE and AZIMUTH are always included.
@@ -578,6 +629,8 @@ namespace GeographicLib {
     unsigned Capabilities() const { return _caps; }
 
     /**
+     * Test what capabilities are available.
+     *
      * @param[in] testcaps a set of bitor'ed GeodesicLineExact::mask values.
      * @return true if the GeodesicLineExact object has all these capabilities.
      **********************************************************************/
@@ -585,6 +638,26 @@ namespace GeographicLib {
       testcaps &= OUT_ALL;
       return (_caps & testcaps) == testcaps;
     }
+
+    /**
+     * The distance or arc length to point 3.
+     *
+     * @param[in] arcmode boolean flag determining the meaning of returned
+     *   value.
+     * @return \e s13 if \e arcmode is false; \e a13 if \e arcmode is true.
+     **********************************************************************/
+    Math::real GenDistance(bool arcmode) const
+    { return Init() ? (arcmode ? _a13 : _s13) : Math::NaN(); }
+
+    /**
+     * @return \e s13, the distance to point 3 (meters).
+     **********************************************************************/
+    Math::real Distance() const { return GenDistance(false); }
+
+    /**
+     * @return \e a13, the arc length to point 3 (degrees).
+     **********************************************************************/
+    Math::real Arc() const { return GenDistance(true); }
     ///@}
 
   };
