@@ -170,8 +170,7 @@ namespace GeographicLib {
         }
       }
     }
-    real height(real lat, real lon, bool gradp,
-                real& grade, real& gradn) const;
+    real height(real lat, real lon) const;
     Geoid(const Geoid&);            // copy constructor not allowed
     Geoid& operator=(const Geoid&); // copy assignment not allowed
   public:
@@ -284,35 +283,9 @@ namespace GeographicLib {
      * The latitude should be in [&minus;90&deg;, 90&deg;].
      **********************************************************************/
     Math::real operator()(real lat, real lon) const {
-      real gradn, grade;
-      return height(lat, lon, false, gradn, grade);
+      return height(lat, lon);
     }
 
-    /// \cond SKIP
-    /**
-     * Compute the geoid height and gradient at a point
-     *
-     * @param[in] lat latitude of the point (degrees).
-     * @param[in] lon longitude of the point (degrees).
-     * @param[out] gradn northerly gradient (dimensionless).
-     * @param[out] grade easterly gradient (dimensionless).
-     * @exception GeographicErr if there's a problem reading the data; this
-     *   never happens if (\e lat, \e lon) is within a successfully cached area.
-     * @return geoid height (meters).
-     *
-     * The latitude should be in [&minus;90&deg;, 90&deg;].  As a result of the
-     * way that the geoid data is stored, the calculation of gradients can
-     * result in large quantization errors.  This is particularly acute for
-     * fine grids, at high latitudes, and for the easterly gradient.  For this
-     * reason, the computation of the gradient is <b>DEPRECATED</b>.  If you
-     * need to compute the direction of the acceleration due to gravity
-     * accurately, you should use GravityModel::Gravity.
-     **********************************************************************/
-    Math::real operator()(real lat, real lon, real& gradn, real& grade) const {
-      return height(lat, lon, true, gradn, grade);
-    }
-    /// \endcond
-
     /**
      * Convert a height above the geoid to a height above the ellipsoid and
      * vice versa.
@@ -330,8 +303,7 @@ namespace GeographicLib {
      **********************************************************************/
     Math::real ConvertHeight(real lat, real lon, real h,
                              convertflag d) const {
-      real gradn, grade;
-      return h + real(d) * height(lat, lon, false, gradn, grade);
+      return h + real(d) * height(lat, lon);
     }
 
     ///@}
@@ -468,15 +440,6 @@ namespace GeographicLib {
     Math::real Flattening() const { return Constants::WGS84_f(); }
     ///@}
 
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
-     **********************************************************************/
-    Math::real InverseFlattening() const
-    { return 1/Constants::WGS84_f(); }
-    /// \endcond
-
     /**
      * @return the default path for geoid data files.
      *
