@@ -205,14 +205,6 @@ namespace GeographicLib {
     static Math::real Flattening()
     { return real(20923713 - 20853810) / 20923713; }
 
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the Airy 1830 ellipsoid.
-     **********************************************************************/
-    static Math::real InverseFlattening() { return 1/Flattening(); }
-    /// \endcond
-
     /**
      * @return \e k0 central scale for the OSGB projection (0.9996012717...).
      *
