@@ -24,6 +24,9 @@ function geographiclib_test
   i = GeodSolve28; if i, n=n+1; fprintf('GeodSolve28 fail: %d\n', i); end
   i = GeodSolve33; if i, n=n+1; fprintf('GeodSolve33 fail: %d\n', i); end
   i = GeodSolve55; if i, n=n+1; fprintf('GeodSolve55 fail: %d\n', i); end
+  i = GeodSolve59; if i, n=n+1; fprintf('GeodSolve59 fail: %d\n', i); end
+  i = GeodSolve61; if i, n=n+1; fprintf('GeodSolve61 fail: %d\n', i); end
+  i = GeodSolve73; if i, n=n+1; fprintf('GeodSolve73 fail: %d\n', i); end
   i = Planimeter0; if i, n=n+1; fprintf('Planimeter0 fail: %d\n', i); end
   i = Planimeter5; if i, n=n+1; fprintf('Planimeter5 fail: %d\n', i); end
   i = Planimeter6; if i, n=n+1; fprintf('Planimeter6 fail: %d\n', i); end
@@ -401,6 +404,35 @@ function n = GeodSolve55
   n = n + assertNaN(s12);
 end
 
+function n = GeodSolve59
+% Check for points close with longitudes close to 180 deg apart.
+  n = 0;
+  [s12, azi1, azi2] = geoddistance(5, 0.00000000000001, 10, 180);
+  n = n + assertEquals(azi1, 0.000000000000035, 1.5e-14);
+  n = n + assertEquals(azi2, 179.99999999999996, 1.5e-14);
+  n = n + assertEquals(s12, 18345191.174332713, 2.5e-9);
+end
+
+function n = GeodSolve61
+% Make sure small negative azimuths are west-going
+  n = 0;
+  [lat2, lon2, azi2] = geodreckon(45, 0, 1e7, -0.000000000000000003, 2);
+  n = n + assertEquals(lat2, 45.30632, 0.5e-5);
+  n = n + assertEquals(lon2, -180, 0.5e-5);
+  n = n + assertEquals(azi2, -180, 0.5e-5);
+end
+
+function n = GeodSolve73
+% Check for backwards from the pole bug reported by Anon on 2016-02-13.
+% This only affected the Java implementation.  It was introduced in Java
+% version 1.44 and fixed in 1.46-SNAPSHOT on 2016-01-17.
+  n = 0;
+  [lat2, lon2, azi2] = geodreckon(90, 10, -1e6, 180);
+  n = n + assertEquals(lat2, 81.04623, 0.5e-5);
+  n = n + assertEquals(lon2, -170, 0.5e-5);
+  n = n + assertEquals(azi2, 0, 0.5e-5);
+end
+
 function n = Planimeter0
 % Check fix for pole-encircling bug found 2011-03-16
   n = 0;
