@@ -31,8 +31,7 @@ int main(int argc, char* argv[]) {
     using namespace GeographicLib;
     typedef Math::real real;
     Utility::set_digits();
-    bool cacheall = false, cachearea = false, verbose = false,
-      cubic = true, gradp = false;
+    bool cacheall = false, cachearea = false, verbose = false, cubic = true;
     real caches, cachew, cachen, cachee;
     std::string dir;
     std::string geoid = Geoid::DefaultGeoidName();
@@ -68,7 +67,7 @@ int main(int argc, char* argv[]) {
       else if (arg == "--haetomsl")
         heightmult = Geoid::ELLIPSOIDTOGEOID;
       else if (arg == "-w")
-        longfirst = true;
+        longfirst = !longfirst;
       else if (arg == "-z") {
         if (++m == argc) return usage(1, true);
         std::string zone = argv[m];
@@ -91,8 +90,6 @@ int main(int argc, char* argv[]) {
         dir = argv[m];
       } else if (arg == "-l")
         cubic = false;
-      else if (arg == "-g")
-        gradp = true;
       else if (arg == "-v")
         verbose = true;
       else if (arg == "--input-string") {
@@ -253,19 +250,8 @@ int main(int argc, char* argv[]) {
                     << Utility::str(height + real(heightmult) * h, 4)
                     << suff << eol;
           } else {
-            if (gradp) {
-            real gradn, grade;
-            real h = g(p.Latitude(), p.Longitude(), gradn, grade);
-            *output << Utility::str(h, 4) << " "
-                    << Utility::str(gradn * 1e6, 2)
-                    << (Math::isnan(gradn) ? " " : "e-6 ")
-                    << Utility::str(grade * 1e6, 2)
-                    << (Math::isnan(grade) ? "" : "e-6")
-                    << eol;
-            } else {
             real h = g(p.Latitude(), p.Longitude());
             *output << Utility::str(h, 4) << eol;
-            }
           }
         }
         catch (const std::exception& e) {
